#include "dialog.h"
#include "ui_dialog.h"
#include "cwd.h"
#include <QDebug>
#include <QDir>
#include <QMessageBox>
#include <QKeyEvent>
#include<QPalette>

#define FKEYCHARS 8


Dialog::Dialog(QWidget *parent)
    : QDialog(parent), ui(new Ui::Dialog)
{
    // The Constructor
    ui->setupUi(this);
    int rc;
    do_cuts = FALSE;
    rc = mycwd.cw_init();//Start connection to CWdaemon
    //qDebug() << "Return code (0 is good) = " << rc;
    tempqso = "";
    oldqso = "";
    serial = 1;
    lcd_cnt = 1; // Reset the counter
    // Check for configuration directory.  Make one if needed.
    hp = cfg_dir_chk();
    load_cfg(); // Load configuration file
    load_qso_nr(); // Loads the QSO number
    ui->logcall_LE->setFocus();
    set_hotkeys();
    tab_init();
    mycwd.cw_send(2, "25"); // Initialize CWdaemon at 25 wpm
    mode_cnt =1;
 }

Dialog::~Dialog()
{
    mycwd.cw_close();
    delete ui;
}

void Dialog::on_pushButton_3_clicked()
{
save_cfg();
}

//-------------------------------------------- Hand keying CW routine -------------------------------------------
void Dialog::on_qso_LE_textChanged(QString cw)
{
    //A key was pressed.
    tempqso = cw;// save the new value from the line edit field
    cw = cw.remove(oldqso);
    // beep contains the letter that was pressed
    msgtype = 1; //text message
    cw.toUpper();
    cw_output(cw);
    QByteArray ba =cw.toUtf8();
    const char *ms = ba.data();
    strcpy(mymsg, ms);
    mycwd.cw_send(msgtype, mymsg);
    oldqso = tempqso;      
}

//-------------------------------------------- Send CW routine -------------------------------------------
void Dialog::send_cw(QString cw){
// A QString that contains the CW to be sent
    do_cuts = TRUE; // Only do cut number substitution when sending CW
    cw = macro_expander(cw);
    do_cuts = FALSE;
    //qDebug() << "CW out = " << cw;
   cw_output(cw);
    msgtype = 1; //text message
    cw.toUpper(); //Convert text to upper case
    QByteArray ba = cw.toUtf8();
    const char *ms = ba.data();
    strcpy(mymsg, ms);
     mycwd.cw_send(msgtype, mymsg);
    }
//--------------------------------------------------------------------------------------------------------------
QString Dialog::macro_expander(QString indata){
    // Macros to expand are; %c, %u, %n, %s, %a, %b

    if(indata.contains(QRegExp("%[Cc]"))){
            // MY CALLSIGN
             if(ui->callsign_LE->text().isEmpty()){ return indata; }
             indata.replace(QRegExp("%[Cc]"), ui->callsign_LE->text());
            }
    if(indata.contains(QRegExp("%[Uu]"))){
            // YOUR CALLSIGN
            if(ui->urcall_LE->text().isEmpty()){ return indata; }
            indata.replace(QRegExp("%[Uu]"), ui->urcall_LE->text());
            }
     if(indata.contains(QRegExp("%[Nn]"))){
            // SERIAL NUMBER
            if(ui->nr_LE->text().isEmpty()){ return indata; }
            // Check for cut numbers
            if(ui->nr_CB->isChecked() && do_cuts){
               QString nv = cut_nums(ui->nr_LE->text());
               indata.replace(QRegExp("%[Nn]"), nv);
            }else
            {
                indata.replace(QRegExp("%[Nn]"), ui->nr_LE->text());
            }
        }
     if(indata.contains(QRegExp("%[Ss]"))){
            // SECTION
            if(ui->sec_LE->text().isEmpty()){ return indata; }
             indata.replace(QRegExp("%[Ss]"), ui->sec_LE->text());
            }
     if(indata.contains(QRegExp("%[Aa]"))){
            // EXCHANGE
            if(ui->x1_LE->text().isEmpty()){ return indata; }
            // Check for cut numbers
            if(ui->exch_CB->isChecked() && do_cuts){
               QString nv = cut_nums(ui->x1_LE->text());
               indata.replace(QRegExp("%[Aa]"), nv);
            }else
            {
                indata.replace(QRegExp("%[Aa]"), ui->x1_LE->text());
            }
            }
      if(indata.contains(QRegExp("%[Bb]"))){
            // MY NAME
            if(ui->x2_LE->text().isEmpty()){ return indata; }
            indata.replace(QRegExp("%[Bb]"), ui->x2_LE->text());
            }
      if(indata.contains(QRegExp("%[Tt]"))){
             // TX RST
             if(ui->txrst_LE->text().isEmpty()){ return indata; }
             // Check for cut numbers
             if(ui->rst_CB->isChecked() && do_cuts){
                QString nv = cut_nums(ui->txrst_LE->text());
                indata.replace(QRegExp("%[Tt]"), nv);
             }else
             {
                 indata.replace(QRegExp("%[Tt]"), ui->txrst_LE->text());
             }
             }


             return indata;
}

void Dialog::on_pushButton_4_clicked()
{
    ui->qso_LE->clear();
}
// ------------------------------------------------------------------------------------------------------

void Dialog::on_callsign_LE_textChanged(QString f)
{
    f = f.toUpper();// Callsigns should be uppercase.
    ui->callsign_LE->setText(f);
    ui->callsign_LBL->setText(f);
     btn_update();
}

void Dialog::on_urcall_LE_textChanged(QString f)
{
    f = f.toUpper();// CW should be uppercase.
    ui->urcall_LE->setText(f);
     btn_update();
}

void Dialog::on_sec_LE_textChanged(QString f)
{
    f = f.toUpper();// CW should be uppercase.
    ui->sec_LE->setText(f);
     btn_update();
}

void Dialog::on_x1_LE_textChanged(QString f)
{
    f = f.toUpper();// CW should be uppercase.
    ui->x1_LE->setText(f);
     btn_update();
}

void Dialog::on_x2_LE_textChanged(QString f)
{
    f = f.toUpper();// CW should be uppercase.
    ui->x2_LE->setText(f);
    btn_update();
}
// ------------------------------------------------------------------------------------------------------
void Dialog::on_f1_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f1_LE->setText(f);
        tmp = "F1 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp); //Expand any macros to the actual text
        ui->f1_PB->setText(tmp);
        //Update the tooltip
        ui->f1_PB->setToolTip(f);
        push_list();// Store to array
}

void Dialog::on_f2_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f2_LE->setText(f);
        tmp = "F2 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f2_PB->setText(tmp);
        ui->f2_PB->setToolTip(f);
         push_list();
}

void Dialog::on_f3_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f3_LE->setText(f);
        tmp = "F3 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f3_PB->setText(tmp);
         ui->f3_PB->setToolTip(f);
         push_list();
}

void Dialog::on_f4_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f4_LE->setText(f);
        tmp = "F4 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f4_PB->setText(tmp);
        ui->f4_PB->setToolTip(f);
        push_list();
}

void Dialog::on_f5_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f5_LE->setText(f);
        tmp = "F5 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f5_PB->setText(tmp);
        ui->f5_PB->setToolTip(f);
         push_list();
}

void Dialog::on_f6_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f6_LE->setText(f);
        tmp = "F6 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f6_PB->setText(tmp);
        ui->f6_PB->setToolTip(f);
         push_list();
}

void Dialog::on_f7_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f7_LE->setText(f);
        tmp = "F7 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f7_PB->setText(tmp);
        ui->f7_PB->setToolTip(f);
         push_list();
}

void Dialog::on_f8_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f8_LE->setText(f);
        tmp = "F8 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f8_PB->setText(tmp);
        ui->f8_PB->setToolTip(f);
         push_list();
}

void Dialog::on_f9_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f9_LE->setText(f);
        tmp = "F9 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f9_PB->setText(tmp);
        ui->f9_PB->setToolTip(f);
         push_list();
}

void Dialog::on_f10_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f10_LE->setText(f);
        tmp = "F10 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f10_PB->setText(tmp);
        ui->f10_PB->setToolTip(f);
         push_list();
}

void Dialog::on_f11_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f11_LE->setText(f);
        tmp = "F11 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f11_PB->setText(tmp);
        ui->f11_PB->setToolTip(f);
         push_list();
}

void Dialog::on_f12_LE_textChanged(QString f)
{
        f = f.toUpper();
        ui->f12_LE->setText(f);
        tmp = "F12 " + f.left(FKEYCHARS);
        tmp = macro_expander(tmp);
        ui->f12_PB->setText(tmp);
        ui->f12_PB->setToolTip(f);
         push_list();
}
//----------------------------------------------------- Save configuration to a file ------------------------
void Dialog::save_cfg(){
    QString cfg_path;
    cfg_path = hp;
    cfg_path.append("eepkeyer.cfg");
    // This will try to create the file if it does not exist
    QFile cfg_file( cfg_path );
// First save the normal program settings
    if(!cfg_file.open(QIODevice::WriteOnly | QIODevice::Text)){
             msgBox.setText("Cannot open ~/.eepkeyer/eepkeyer.cfg file.");
             msgBox.exec();
             return;
         }
    QTextStream ts( &cfg_file );
    // Data to save
    ts << "speed " << ui->speed_LE->text() << endl;
    ts << "speed1 " << ui->speed1_LE->text() << endl;
    ts << "speed2 " << ui->speed2_LE->text() << endl;
    ts << "speed3 " << ui->speed3_LE->text() << endl;
    ts << "speed4 " << ui->speed4_LE->text() << endl;

    // Macros
    ts << "%c:"<< ui->callsign_LE->text() << endl;
    //ts << "%n:" << ui->nr_LE->text() << endl;// QSO number saved in another file
    ts << "%s:" << ui->sec_LE->text() << endl;
    ts << "%a:" << ui->x1_LE->text() << endl;
    ts << "%b:" << ui->x2_LE->text() << endl;
    // F-keys
        // Defaults
    ts << "DEFAULT:" << endl;    
       if(c1_d.empty()){
                c1_d << "empty" << "" << "" << "" << ""<< ""<< ""<< ""<< ""<< ""<< ""<< "";
    }
    ts << "F1:" << c1_d[0] << endl;
    ts << "F2:" <<c1_d[1] << endl;
    ts << "F3:" << c1_d[2] << endl;
    ts << "F4:" << c1_d[3] << endl;
    ts << "F5:" << c1_d[4] << endl;
    ts << "F6:" << c1_d[5] << endl;
    ts << "F7:" << c1_d[6] << endl;
    ts << "F8:" << c1_d[7] << endl;
    ts << "F9:" << c1_d[8] << endl;
    ts << "F10:" << c1_d[9] << endl;
    ts << "F11:" << c1_d[10] << endl;
    ts << "F12:" << c1_d[11] << endl;

    ts << "CQ:" << endl;
    if(c2_d.empty()){
                c2_d << "empty" << "" << "" << "" << ""<< ""<< ""<< ""<< ""<< ""<< ""<< "";
    }
    ts << "F1:" << c2_d[0] << endl;
    ts << "F2:" <<c2_d[1] << endl;
    ts << "F3:" << c2_d[2] << endl;
    ts << "F4:" << c2_d[3] << endl;
    ts << "F5:" << c2_d[4] << endl;
    ts << "F6:" << c2_d[5] << endl;
    ts << "F7:" << c2_d[6] << endl;
    ts << "F8:" << c2_d[7] << endl;
    ts << "F9:" << c2_d[8] << endl;
    ts << "F10:" << c2_d[9] << endl;
    ts << "F11:" << c2_d[10] << endl;
    ts << "F12:" << c2_d[11] << endl;

    ts << "SNP:" << endl;
    if(c3_d.empty()){
        c3_d << "empty" << "" << "" << "" << ""<< ""<< ""<< ""<< ""<< ""<< ""<< "";
    }
    ts << "F1:" << c3_d[0] << endl;
    ts << "F2:" <<c3_d[1] << endl;
    ts << "F3:" << c3_d[2] << endl;
    ts << "F4:" << c3_d[3] << endl;
    ts << "F5:" << c3_d[4] << endl;
    ts << "F6:" << c3_d[5] << endl;
    ts << "F7:" << c3_d[6] << endl;
    ts << "F8:" << c3_d[7] << endl;
    ts << "F9:" << c3_d[8] << endl;
    ts << "F10:" << c3_d[9] << endl;
    ts << "F11:" << c3_d[10] << endl;
    ts << "F12:" << c3_d[11] << endl;

    ts << "TAB ORDER:" << endl;
    ts << "TAB 1: " << ui->lcdNumber_1->value() << endl;
    //ts << "TAB 2: " << ui->lcdNumber_2->value() << endl;
    ts << "TAB 3: " << ui->lcdNumber_3->value() << endl;
    ts << "TAB 4: " << ui->lcdNumber_4->value() << endl;
    ts << "TAB 5: " << ui->lcdNumber_5->value() << endl;
    ts << "TAB 6: " << ui->lcdNumber_6->value() << endl;
    //ts << "TAB 7: " << ui->lcdNumber_7->value() << endl;

    ts << "CUT 0: " << ui->zero_CB->isChecked()   << endl;
    ts << "CUT 1: " << ui->one_CB->isChecked()  << endl;
    ts << "CUT 2: " << ui->two_CB->isChecked()  << endl;
    ts << "CUT 3: " << ui->three_CB->isChecked()  << endl;
    ts << "CUT 4: " << ui->four_CB->isChecked()  << endl;
    ts << "CUT 5: " << ui->five_CB->isChecked()  << endl;
    ts << "CUT 6: " << ui->six_CB->isChecked()  << endl;
    ts << "CUT 7: " << ui->seven_CB->isChecked()  << endl;
    ts << "CUT 8: " << ui->eight_CB->isChecked()  << endl;
    ts << "CUT 9: " << ui->nine_CB->isChecked()  << endl;

    ts << "C_NUM: " << ui->nr_CB->isChecked()  << endl;
    ts << "C_EXCH: " << ui->exch_CB->isChecked()  << endl;
    ts << "C_RST: " << ui->rst_CB->isChecked()  << endl;
    ts << "HAMLIB: " << ui->hamlib_CB->isChecked()  << endl;

    // Close the file when done
    cfg_file.close();
    save_qso_nr();

    msgBox.setIcon(QMessageBox::Information);
    msgBox.setText("Settings saved.");
    msgBox.exec();
}
//---------------------------------------------------- Load saved configuration from a file -----------------------------
void Dialog::load_cfg(){
     QString cfg_path;
     cfg_path = hp;
     cfg_path.append("eepkeyer.cfg");
     QFile cfg_file( cfg_path );
    // Check to see if the ~/.eepkeyer/eepkeyer.cfg file exists
    if(cfg_file.exists(cfg_path)){
        // Read settings from eepkeyer.cfg file
        //qDebug() << "Config file found" << endl;
        rd = "";
        if(!cfg_file.open(QIODevice::ReadOnly | QIODevice::Text)){
            msgBox.setIcon(QMessageBox::Critical);
            msgBox.setText(tr("Configuration file exists but cannot open it."));
            msgBox.exec();
            return;
        }
        QTextStream rs( &cfg_file );
        // Read each line and parse the data
        while(!rs.atEnd()){
            rd = rs.readLine();
            // --------- Read macros -----------
             if(rd.contains("speed ")){
                rd.remove("speed ");
                ui->speed_LE->setText(rd);
              }
               if(rd.contains("speed1 ")){
                rd.remove("speed1 ");
                ui->speed1_LE->setText(rd);
                 ui->speed1_PB->setText(rd);
              }
               if(rd.contains("speed2 ")){
                rd.remove("speed2 ");
                ui->speed2_LE->setText(rd);
                ui->speed2_PB->setText(rd);
              }
             if(rd.contains("speed3 ")){
                rd.remove("speed3 ");
                ui->speed3_LE->setText(rd);
                ui->speed3_PB->setText(rd);
              }
             if(rd.contains("speed4 ")){
                rd.remove("speed4 ");
                ui->speed4_LE->setText(rd);
                ui->speed4_PB->setText(rd);
              }

            if(rd.contains("%c:")){
                rd.remove("%c:");                
                ui->callsign_LE->setText(rd);
              }

            if(rd.contains("%s:")){
                rd.remove("%s:");
                ui->sec_LE->setText(rd);
              }
            if(rd.contains("%a:")){
                rd.remove("%a:");
                ui->x1_LE->setText(rd);
              }
            if(rd.contains("%b:")){
                rd.remove("%b:");
                ui->x2_LE->setText(rd);
              }
            // ---------- Now read in F-keys ------------
            // There are 3 Banks of F-keys to read in
            if(rd.contains("DEFAULT:")){
                // Enable default read
                c1 = TRUE;
                c2 = FALSE;
                c3 = FALSE;
              }
            if(rd.contains("CQ:")){
                // Enable default read
                c1 = FALSE;
                c2 = TRUE;
                c3 = FALSE;
              }
            if(rd.contains("SNP:")){
                // Enable default read
                c1 = FALSE;
                c2 = FALSE;
                c3 = TRUE;
              }

            if(rd.contains("F1:")){
                rd.remove("F1:");
                stuffit();
              }
            if(rd.contains("F2:")){
                rd.remove("F2:");
                stuffit();
              }
            if(rd.contains("F3:")){
                rd.remove("F3:");
                stuffit();
              }
            if(rd.contains("F4:")){
                rd.remove("F4:");
                stuffit();
              }
            if(rd.contains("F5:")){
                rd.remove("F5:");
                stuffit();
              }
            if(rd.contains("F6:")){
                rd.remove("F6:");
                stuffit();
              }
            if(rd.contains("F7:")){
                rd.remove("F7:");
                stuffit();
              }
            if(rd.contains("F8:")){
                rd.remove("F8:");
                stuffit();
              }
            if(rd.contains("F9:")){
                rd.remove("F9:");
                stuffit();
              }
            if(rd.contains("F10:")){
                rd.remove("F10:");
                stuffit();
              }
            if(rd.contains("F11:")){
                rd.remove("F11:");
                stuffit();
              }
            if(rd.contains("F12:")){
                rd.remove("F12:");
                stuffit();
              }
           if(rd.contains ("TAB 1: ")){
                rd.remove("TAB 1: ");
                ui->lcdNumber_1->display(rd);             
           }

           if(rd.contains ("TAB 3: ")){
                rd.remove("TAB 3: ");
                ui->lcdNumber_3->display(rd);
           }
           if(rd.contains ("TAB 4: ")){
                rd.remove("TAB 4: ");
                ui->lcdNumber_4->display(rd);
           }
            if(rd.contains ("TAB 5: ")){
                rd.remove("TAB 5: ");
                ui->lcdNumber_5->display(rd);
           }
           if(rd.contains ("TAB 6: ")){
                rd.remove("TAB 6: ");
                ui->lcdNumber_6->display(rd);
           }
           if(rd.contains("CUT 0: ")){
               rd.remove("CUT 0: ");
               if(rd == "1"){
               ui->zero_CB->setChecked( TRUE );
           }else{
               ui->zero_CB->setChecked( FALSE );
             }
       }
           if(rd.contains("CUT 1: ")){
               rd.remove("CUT 1: ");
               if(rd == "1"){
               ui->one_CB->setChecked( TRUE );
           }else{
               ui->one_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("CUT 2: ")){
               rd.remove("CUT 2: ");
               if(rd == "1"){
               ui->two_CB->setChecked( TRUE );
           }else{
               ui->two_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("CUT 3: ")){
               rd.remove("CUT 3: ");
               if(rd == "1"){
               ui->three_CB->setChecked( TRUE );
           }else{
               ui->three_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("CUT 4: ")){
               rd.remove("CUT 4: ");
               if(rd == "1"){
               ui->four_CB->setChecked( TRUE );
           }else{
               ui->four_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("CUT 5: ")){
               rd.remove("CUT 5: ");
               if(rd == "1"){
               ui->five_CB->setChecked( TRUE );
           }else{
               ui->five_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("CUT 6: ")){
               rd.remove("CUT 6: ");
               if(rd == "1"){
               ui->six_CB->setChecked( TRUE );
           }else{
               ui->six_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("CUT 7: ")){
               rd.remove("CUT 7: ");
               if(rd == "1"){
               ui->seven_CB->setChecked( TRUE );
           }else{
               ui->seven_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("CUT 8: ")){
               rd.remove("CUT 8: ");
               if(rd == "1"){
               ui->eight_CB->setChecked( TRUE );
           }else{
               ui->eight_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("CUT 9: ")){
               rd.remove("CUT 9: ");
               if(rd == "1"){
               ui->nine_CB->setChecked( TRUE );
           }else{
               ui->nine_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("C_NUM: ")){
               rd.remove("C_NUM: ");
               if(rd == "1"){
               ui->nr_CB->setChecked( TRUE );
           }else{
               ui->nr_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("C_EXCH: ")){
               rd.remove("C_EXCH: ");
               if(rd == "1"){
               ui->exch_CB->setChecked( TRUE );
           }else{
               ui->exch_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("C_RST: ")){
               rd.remove("C_RST: ");
               if(rd == "1"){
               ui->rst_CB->setChecked( TRUE );
           }else{
               ui->rst_CB->setChecked( FALSE );
             }
             }
           if(rd.contains("HAMLIB: ")){
               rd.remove("HAMLIB: ");
               if(rd == "1"){
               ui->hamlib_CB->setChecked( TRUE );
           }else{
               ui->hamlib_CB->setChecked( FALSE );
             }
             }

        }        
        pop_list(); // Populate the active bank F-keys with values
        tab_init(); // Initialize the tab variables
        get_max_tab(); // Find out how many tabs are set
        tab_order(); // Set the order
        cfg_file.close(); // Clean up and close the config file

        return;
    }
   // An existing config file was not found so load default values.
   msgBox.setIcon(QMessageBox::Warning);
   msgBox.setText("No config file found.  Using defaults.");
   msgBox.exec();
   dflt_cfg();
}

  void Dialog::pop_list(){
      // what F-key bank is selected by the combo box?
      int idx;
      idx = ui->bank_CB->currentIndex();
      QList<QString> tmp;
      tmp.clear();
    if(idx == 0){
        // Load Default bank
        tmp = c1_d;
    }
     if(idx == 1){
        // Load Default bank
        tmp = c2_d;
    }
      if(idx == 2){
        // Load Default bank
        tmp = c3_d;
    }
     ui->f1_LE->setText(tmp[0]);    
     ui->f2_LE->setText(tmp[1]);
     ui->f3_LE->setText(tmp[2]);
     ui->f4_LE->setText(tmp[3]);
     ui->f5_LE->setText(tmp[4]);
     ui->f6_LE->setText(tmp[5]);
     ui->f7_LE->setText(tmp[6]);
     ui->f8_LE->setText(tmp[7]);
     ui->f9_LE->setText(tmp[8]);
     ui->f10_LE->setText(tmp[9]);
     ui->f11_LE->setText(tmp[10]);
     ui->f12_LE->setText(tmp[11]);
   }

  void Dialog::btn_update(){
          // what F-key bank is selected by the combo box?
      // Populate the F-key buttons from stored array
      //qDebug() << "btn_update ";
      if(c1_d.isEmpty()){
          return;
      }
      int idx;
      QList<QString> tmp;
      idx = ui->bank_CB->currentIndex();
    if(idx == 0){
        // Load Default bank
        tmp = c1_d;
    }
     if(idx == 1){
        // Load Default bank
        tmp = c2_d;
    }
      if(idx == 2){
        // Load Default bank
        tmp = c3_d;
    }

      tmp[0] = macro_expander(tmp[0]);
      ui->f1_PB->setText( ("F1 " +  tmp[0].left(FKEYCHARS)));
      ui->f1_PB->setToolTip(tmp[0].remove("F1 "));

      tmp[1] = macro_expander(tmp[1]);
      ui->f2_PB->setText(("F2 " + tmp[1].left(FKEYCHARS)));
      ui->f2_PB->setToolTip(tmp[1].remove("F2 "));

     tmp[2] = macro_expander(tmp[2]);
     ui->f3_PB->setText(("F3 " + tmp[2].left(FKEYCHARS)));
      ui->f3_PB->setToolTip(tmp[2].remove("F3 "));

     tmp[3] = macro_expander(tmp[3]);
     ui->f4_PB->setText(("F4 " + tmp[3].left(FKEYCHARS)));
      ui->f4_PB->setToolTip(tmp[3].remove("F4 "));

     tmp[4] = macro_expander(tmp[4]);
     ui->f5_PB->setText(("F5 " + tmp[4].left(FKEYCHARS)));
      ui->f5_PB->setToolTip(tmp[4].remove("F5 "));

     tmp[5] = macro_expander(tmp[5]);
     ui->f6_PB->setText(("F6 " + tmp[5].left(FKEYCHARS)));
      ui->f6_PB->setToolTip(tmp[5].remove("F6 "));

     tmp[6] = macro_expander(tmp[6]);
     ui->f7_PB->setText(("F7 " + tmp[6].left(FKEYCHARS)));
     ui->f7_PB->setToolTip(tmp[6].remove("F7 "));

     tmp[7] = macro_expander(tmp[7]);
     ui->f8_PB->setText(("F8 " + tmp[7].left(FKEYCHARS)));
     ui->f8_PB->setToolTip(tmp[7].remove("F8 "));

     tmp[8] = macro_expander(tmp[8]);
     ui->f9_PB->setText(("F9 " + tmp[8].left(FKEYCHARS)));
     ui->f9_PB->setToolTip(tmp[8].remove("F9 "));

     tmp[9] = macro_expander(tmp[9]);
     ui->f10_PB->setText(("F10 " + tmp[9].left(FKEYCHARS)));
     ui->f10_PB->setToolTip(tmp[9].remove("F10 "));

     tmp[10] = macro_expander(tmp[10]);
     ui->f11_PB->setText(("F11 " + tmp[10].left(FKEYCHARS)));
     ui->f11_PB->setToolTip(tmp[10].remove("F11 "));

     tmp[11] = macro_expander(tmp[11]);
     ui->f12_PB->setText(("F12 " + tmp[11].left(FKEYCHARS)));
     ui->f12_PB->setToolTip(tmp[11].remove("F12 "));

tmp.clear();
set_hotkeys();
  }

   void Dialog::push_list(){
      // Stuff whats on the GUI into memory
       // what F-key bank is selected by the combo box?
      int idx;
      idx = ui->bank_CB->currentIndex();
      QList<QString> tmp;
    if(idx == 0){
        //  Default bank

     c1_d[0] = ui->f1_LE->text();
     c1_d[1] = ui->f2_LE->text();
     c1_d[2] = ui->f3_LE->text();
     c1_d[3] = ui->f4_LE->text();
     c1_d[4] = ui->f5_LE->text();
     c1_d[5] = ui->f6_LE->text();
     c1_d[6] = ui->f7_LE->text();
     c1_d[7] = ui->f8_LE->text();
     c1_d[8] = ui->f9_LE->text();
     c1_d[9] = ui->f10_LE->text();
     c1_d[10] = ui->f11_LE->text();
     c1_d[11] = ui->f12_LE->text();
    }
     if(idx == 1){
        // CQ bank

     c2_d[0] = ui->f1_LE->text();
     c2_d[1] = ui->f2_LE->text();
     c2_d[2] = ui->f3_LE->text();
     c2_d[3] = ui->f4_LE->text();
     c2_d[4] = ui->f5_LE->text();
     c2_d[5] = ui->f6_LE->text();
     c2_d[6] = ui->f7_LE->text();
     c2_d[7] = ui->f8_LE->text();
     c2_d[8] = ui->f9_LE->text();
     c2_d[9] = ui->f10_LE->text();
     c2_d[10] = ui->f11_LE->text();
     c2_d[11] = ui->f12_LE->text();
    }
      if(idx == 2){
        // S&P bank

     c3_d[0] = ui->f1_LE->text();
     c3_d[1] = ui->f2_LE->text();
     c3_d[2] = ui->f3_LE->text();
     c3_d[3] = ui->f4_LE->text();
     c3_d[4] = ui->f5_LE->text();
     c3_d[5] = ui->f6_LE->text();
     c3_d[6] = ui->f7_LE->text();
     c3_d[7] = ui->f8_LE->text();
     c3_d[8] = ui->f9_LE->text();
     c3_d[9] = ui->f10_LE->text();
     c3_d[10] = ui->f11_LE->text();
     c3_d[11] = ui->f12_LE->text();
    }

set_hotkeys();

   }

void Dialog::stuffit(){
      if(c1){
        c1_d << rd;
        }
      if(c2){
         c2_d << rd;
        }
      if(c3){
         c3_d << rd;
        }
    }


void Dialog::on_bank_CB_currentIndexChanged(int index)
{
     mode_cnt =1;
     //qDebug() << "DEBUG: bank_CB_currentIndexChanged" << endl;
    if(index == 0){
        //Default F-keys
        default_labels();
    }
    if(index == 1){
        //CQ F-keys
        cq_labels();
    }
    if(index == 2){
        // SNP F-keys
        snp_labels();
        //QPalette p;
    }
    index = 0;
    pop_list();
    btn_update();
    set_hotkeys();
}

void Dialog::dflt_cfg(){
    ui->callsign_LE->setText("K6EEP");
    ui->nr_LE->setText("001");
    tab_init(); // Making sure the pointers are initialized
    le[1] = tabs[1];
    ui->lcdNumber_1->display(1); //At least one tab has to be set so using callsign first
    save_cfg();
}
//------------------------------------- Tab Order ---------------------------------------

//------------------------------------------ Send the F-key CW -------------------------------------
void Dialog::on_f1_PB_clicked()
{
    cw_out = get_msg(0);
    send_cw(cw_out);
}


void Dialog::on_f2_PB_clicked()
{
    cw_out = get_msg(1);
    send_cw(cw_out);
}

void Dialog::on_f3_PB_clicked()
{
    cw_out = get_msg(2);
    send_cw(cw_out);
}

void Dialog::on_f4_PB_clicked()
{
   cw_out = get_msg(3);
    send_cw(cw_out);
}

void Dialog::on_f5_PB_clicked()
{
    cw_out = get_msg(4);
    send_cw(cw_out);
}

void Dialog::on_f6_PB_clicked()
{
   cw_out = get_msg(5);
    send_cw(cw_out);
}

void Dialog::on_f7_PB_clicked()
{
  cw_out = get_msg(6);
    send_cw(cw_out);
}

void Dialog::on_f8_PB_clicked()
{
   cw_out = get_msg(7);
    send_cw(cw_out);
}

void Dialog::on_f9_PB_clicked()
{
    cw_out = get_msg(8);
    send_cw(cw_out);
}

void Dialog::on_f10_PB_clicked()
{
  cw_out = get_msg(9);
    send_cw(cw_out);
}

void Dialog::on_f11_PB_clicked()
{
   cw_out = get_msg(10);
    send_cw(cw_out);
}

void Dialog::on_f12_PB_clicked()
{
    cw_out = get_msg(11);
    send_cw(cw_out);
}

QString Dialog::get_msg(int f){
    // Check the F-key bank in use
    // f is the F-key index #
    int idx;
    idx = ui->bank_CB->currentIndex();
      if(idx == 0){
             // Load Default bank
         return c1_d[f];
        }
     if(idx == 1){
             // Load CQ bank
            return c2_d[f];
        }
      if(idx == 2){
            // Load S&P bank
            return c3_d[f];
        }
      return "error"; // It should never get here
}



void Dialog::on_tab_set_PB_clicked()
{
    tab_order();
     //  int v = ui->lcdNumber_7->value(); //Get a LCD value
    // lcd_cnt holds current counter
}

void Dialog::on_reset_PB_clicked()
{
    // Set all LCD's to zero
    ui->lcdNumber_1->display(0);// Set the number

    ui->lcdNumber_3->display(0);// Set the number
    ui->lcdNumber_4->display(0);// Set the number
    ui->lcdNumber_5->display(0);// Set the number
    ui->lcdNumber_6->display(0);// Set the number

    lcd_cnt = 0; // Reset the counter
}

void Dialog::on_A_PB_clicked()
{
    //Check current count
    // If counter is at 0 then you can edit it
    if( ui->lcdNumber_1->value() == 0){
       if(lcd_cnt < 7){
        lcd_cnt++;
         ui->lcdNumber_1->display(lcd_cnt);
    }
}
}



void Dialog::on_C_PB_clicked()
{
    if( ui->lcdNumber_3->value() == 0){

     if(lcd_cnt < 7){
            lcd_cnt++;
             ui->lcdNumber_3->display(lcd_cnt);
        }
    }
}

void Dialog::on_D_PB_clicked()
{
    if( ui->lcdNumber_4->value() == 0){

     if(lcd_cnt < 7){
         lcd_cnt++;
         ui->lcdNumber_4->display(lcd_cnt);
        }
    }
}

void Dialog::on_E_PB_clicked()
{
    if( ui->lcdNumber_5->value() == 0){

        if(lcd_cnt < 7){
            lcd_cnt++;
            ui->lcdNumber_5->display(lcd_cnt);
        }
    }
}

void Dialog::on_F_PB_clicked()
{
    if( ui->lcdNumber_6->value() == 0){

        if(lcd_cnt < 7){
            lcd_cnt++;
            ui->lcdNumber_6->display(lcd_cnt);
        }
    }
}




void Dialog::tab_order(){
    // This will be complicated.  It is a linked list of sorts 
   // Outer loop 1-7
    int max_v = 0;
     //-----------------------------
    for(int i = 1; i<6; i++){
        int v =(qlcd[i])->value();
        if(v > 0){
            if(v > max_v){
                max_v = v;
            }
            le[v] = tabs[i];
        }
   }    
    if(max_v == 0){
        //No tab values set so return
        QMessageBox msgbox;
        msgbox.setIcon(QMessageBox::Warning);
        msgbox.setText(tr("No TAB order defined"));
        msgbox.setInformativeText(tr("Set at least one field under Logger Setup"));
        msgbox.exec();
        return;
    }
    for(int j = 1; j < max_v; j++){
          setTabOrder(le[j], le[j+1]);
      }
    le[1]->setFocus();
    setTabOrder(le[max_v], ui->logger_PB);
    }

void Dialog::on_logger_PB_clicked()
{
    if (ui->logcall_LE->text() == NULL)
    {
        QMessageBox msgbox;
        msgbox.setIcon(QMessageBox::Warning);
        msgbox.setText( tr("You need a callsign to log QSO") );
        msgbox.exec();
        return;
    }
    //qDebug() << "DEBUG: logger_PB pressed" << endl;
    logit(); // Send QSO to Xlog
    // Bump the QSO count up one and update GUI;
    serial++;
    QString m;
    m = QString("%1")
             .arg(serial, 3, 10, QLatin1Char( '0' ));
    ui->nr_LE->setText(m); // If you change a macro you need to update F-key PBs
     btn_update();
     // Clear GUI log fields that have tab stops assigned to them
     clr_log();
     // Clear old sent data
     ui->cwout_LE->clear();
    // Reset focus to first field
     le[1]->setFocus();
}

void Dialog::on_nr_LE_textChanged(QString nr)
{
    bool ok;
    serial = nr.toInt(&ok, 10);
    save_qso_nr();// Save new QSO number to file
    btn_update();
}


void Dialog::cw_output(QString c)
{
    c.toUpper();
    QString o;
    o = ui->cwout_LE->text();
    o.append(c);
    ui->cwout_LE->setText(o);
}

void Dialog::clr_log()
{
     if( ui->lcdNumber_1->value() != 0){
       ui->logcall_LE->clear();
     }
       if( ui->lcdNumber_3->value() != 0){
       ui->txrst_LE->clear();
     }
     if( ui->lcdNumber_4->value() != 0){
       ui->rxrst_LE->clear();
     }
     if( ui->lcdNumber_5->value() != 0){
       ui->number_LE->clear();
     }
     if( ui->lcdNumber_6->value() != 0){
       ui->exch1_LE->clear();
     }
}

void Dialog::get_max_tab()
{
    int d;
    d = ui->lcdNumber_1->value();
    if(lcd_cnt < d)
    {
        lcd_cnt = d;
    }

    d = ui->lcdNumber_3->value();
    if(lcd_cnt < d)
    {
        lcd_cnt = d;
    }
    d = ui->lcdNumber_4->value();
    if(lcd_cnt < d)
    {
        lcd_cnt = d;
    }
    d = ui->lcdNumber_5->value();
    if(lcd_cnt < d)
    {
        lcd_cnt = d;
    }
    d = ui->lcdNumber_6->value();
    if(lcd_cnt < d)
    {
        lcd_cnt = d;
    }

    //qDebug() << "lcd_count = " << lcd_cnt;
}

void Dialog::tab_init()
{
    // If arrays do not get initialized, kaboom!
    tabs[1] = ui->logcall_LE;
    tabs[2] = ui->txrst_LE;
    tabs[3] = ui->rxrst_LE;
    tabs[4] = ui->number_LE;
    tabs[5] = ui->exch1_LE;

    qlcd[1] = ui->lcdNumber_1;
    qlcd[2] = ui->lcdNumber_3;
    qlcd[3] = ui->lcdNumber_4;
    qlcd[4] = ui->lcdNumber_5;
    qlcd[5] = ui->lcdNumber_6;

}

void Dialog::logit()
{
   // Collect data to send to Xlog logbook
    // Stuff everything into a big char[1024] array called logmsg
    QString ftmp;
    const char * mm;
    // Name of program sending data to Xlog
    strcpy(logmsg, "program:eepKeyer\1");
    // Callsign to log
    ftmp = ui->logcall_LE->displayText();
    mm = qtostr(ftmp);
    if(!ftmp.isEmpty()){
        strcat(logmsg, "call:");
        strcat(logmsg, mm);
        strcat(logmsg, "\1");
    }
    ftmp = "";
    // Mode of qso
    ftmp = ui->mode_CB->currentText();
    mm = qtostr(ftmp);
     if(!ftmp.isEmpty()){
        strcat(logmsg, "mode:");
        // Set mode in Xlog via hamlib
        if(ui->hamlib_CB->isChecked()){
            strcat(logmsg, "HAMLIB");
        }else{
        strcat(logmsg, mm);
        }
        strcat(logmsg, "\1");
    }
       ftmp = "";
 // RST TX
    ftmp = ui->txrst_LE->displayText();
    mm = qtostr(ftmp);
    if(!ftmp.isEmpty()){
        strcat(logmsg, "tx:");
        strcat(logmsg, mm);
        strcat(logmsg, "\1");
    }
    ftmp = "";
    // RST RX
    ftmp = ui->rxrst_LE->displayText();
   mm = qtostr(ftmp);
    if(!ftmp.isEmpty()){
        strcat(logmsg, "rx:");
        strcat(logmsg, mm);
        strcat(logmsg, "\1");
    }
    ftmp = "";
     // free1:  is the serial number
    ftmp = ui->number_LE->displayText();
   mm = qtostr(ftmp);
    if(!ftmp.isEmpty()){
        strcat(logmsg, "free1:");
        strcat(logmsg, mm);
        strcat(logmsg, "\1");
    }
    ftmp = "";
    // free2:  is the exchange
   ftmp = ui->exch1_LE->displayText();
   mm = qtostr(ftmp);
    if(!ftmp.isEmpty()){
        strcat(logmsg, "free2:");
        strcat(logmsg, mm);
        strcat(logmsg, "\1");
    }
    ftmp = "";   
    //This routine is to allow Xlog to set the frequency via hamlib
    if(ui->hamlib_CB->isChecked()){
        strcat(logmsg, "mhz:");
        strcat(logmsg, "HAMLIB");
        strcat(logmsg, "\1");
    }
    ftmp = "";
// That should be everything.  Xlog takes care of date/time automatically.

    if (xl.sendit(logmsg) == 1){
        // ERROR Routine
        QMessageBox::warning( this, "eepKeyer", tr("Could not send to Xlog program") );
    }
}

void Dialog::dupe_chk()
{
 QString ftmp;
    const char * mm;
    // Name of program sending data to Xlog  
    strcpy(logmsg, "program:eepKeyer\1");

    // Callsign to log
    ftmp = ui->logcall_LE->displayText();
    mm = qtostr(ftmp);
    if(!ftmp.isEmpty()){
        strcat(logmsg, "call:");
        strcat(logmsg, mm);
        strcat(logmsg, "\1");
    }else{
        return;
    }
     ftmp = "";
        strcat(logmsg, "?:");
        strcat(logmsg, "\1");
        //qDebug() << "DEBUG: 1" << logmsg << endl;
    if (xl.sendit(logmsg) == 1){
        // ERROR Routine
        QMessageBox::warning( this, "eepKeyer", tr("Could not send to Xlog program") );
    }
}

const char* Dialog::qtostr(QString qs)
{
    QByteArray ba =qs.toUtf8();
    const char *ms = ba.data();
    return ms;
}


void Dialog::cq_labels()
{
    ui->f1_LBL->setText("CQ test de me");
    ui->f2_LBL->setText("Reply to a call");
    ui->f3_LBL->setText("End contact");
}

void Dialog::snp_labels()
{
    ui->f1_LBL->setText("Send mycall");
    ui->f2_LBL->setText("Send exchange");
    ui->f3_LBL->setText("");
}

void Dialog::default_labels()
{
    ui->f1_LBL->setText("");
    ui->f2_LBL->setText("");
    ui->f3_LBL->setText("");
}



void Dialog::on_pushButton_2_clicked()
{
    // Abort sending CW
    mycwd.cw_send(4, mymsg);// The #4 is the ABORT command

}


void Dialog::set_hotkeys()
{
    ui->f1_PB->setShortcut(QKeySequence(tr("F1")));
    ui->f2_PB->setShortcut(QKeySequence(tr("F2")));
    ui->f3_PB->setShortcut(QKeySequence(tr("F3")));
    ui->f4_PB->setShortcut(QKeySequence(tr("F4")));
    ui->f5_PB->setShortcut(QKeySequence(tr("F5")));
    ui->f6_PB->setShortcut(QKeySequence(tr("F6")));
    ui->f7_PB->setShortcut(QKeySequence(tr("F7")));
    ui->f8_PB->setShortcut(QKeySequence(tr("F8")));
    ui->f9_PB->setShortcut(QKeySequence(tr("F9")));
    ui->f10_PB->setShortcut(QKeySequence(tr("F10")));
    ui->f11_PB->setShortcut(QKeySequence(tr("F11")));
    ui->f12_PB->setShortcut(QKeySequence(tr("F12")));
}


void Dialog::on_speed_LE_textChanged(QString )
{
    const char *ms = qtostr(ui->speed_LE->text());
    strcpy(mymsg, ms);
    mycwd.cw_send(2, mymsg);
}

void Dialog::on_speed1_PB_clicked()
{
    ui->speed_LE->setText(ui->speed1_PB->text());
 }


void Dialog::on_speed2_PB_clicked()
{
   ui->speed_LE->setText(ui->speed2_PB->text());
}

void Dialog::on_speed3_PB_clicked()
{
   ui->speed_LE->setText(ui->speed3_PB->text());
}

void Dialog::on_speed4_PB_clicked()
{
   ui->speed_LE->setText(ui->speed4_PB->text());
}

void Dialog::on_logcall_LE_textEdited(QString f)
{
    f = f.toUpper();// Callsigns should be uppercase for readability.
    ui->urcall_LE->setText(f);
    btn_update();
    dupe_chk();
}

void Dialog::on_speed1_LE_textEdited(QString q )
{
    ui->speed1_PB->setText(q);
}

void Dialog::on_speed2_LE_textEdited(QString q )
{
    ui->speed2_PB->setText(q);
}

void Dialog::on_speed3_LE_textEdited(QString q )
{
    ui->speed3_PB->setText(q);
}

void Dialog::on_speed4_LE_textEdited(QString q )
{
    ui->speed4_PB->setText(q);
}

void Dialog::on_logcall_LE_lostFocus()
{
    // After you leave the callsign entry field it is capitalized
    QString lcl = ui->logcall_LE->text();
    lcl = lcl.toUpper();
    ui->logcall_LE->setText(lcl);
}

void Dialog::save_qso_nr()
{
    //qDebug() << "DEBUG: save qso nr" << endl;
    QString qso_nr_path;
    qso_nr_path = hp;
    qso_nr_path.append("qso_nr.cfg");
    // Now save the qso number

            QFile qso_file( qso_nr_path );
        // Now save the current QSO number
            if(!qso_file.open(QIODevice::WriteOnly | QIODevice::Text)){
                     msgBox.setText("Cannot open ~/.eepkeyer/qso_nr.cfg file.");
                     msgBox.exec();
                     return;
                 }
            QTextStream qts( &qso_file );
            qts << "QSO Number: " << ui->nr_LE->text() << endl;
            qso_file.close();
}
void Dialog::load_qso_nr()
{
    QString qso_nr_path;
    qso_nr_path = hp;
    qso_nr_path.append("qso_nr.cfg");
    QFile qso_nr_file(qso_nr_path);
    // Check to see if the ~/.eepkeyer/qso_nr.cfg file exists
    if(qso_nr_file.exists(qso_nr_path)){
        // Read settings from qso_nr.cfg file
        //qDebug() << "DEBUG: Loading QSO number" << endl;
        rd = ""; // Probably overkill
        if(!qso_nr_file.open(QIODevice::ReadOnly | QIODevice::Text)){
            ui->nr_LE->setText("001"); // Set to a default value of one and return
            return;
        }
        QTextStream rs( &qso_nr_file );
        // Read each line and parse the data
        while(!rs.atEnd()){
            rd = rs.readLine();
            // --------- Read macros -----------

             if(rd.contains("QSO Number: ")){
                 rd.remove("QSO Number: ");
                 ui->nr_LE->setText(rd);
                 bool ok;
                 serial = rd.toInt(&ok, 10);
               }
         }
    }
}

QString Dialog::cfg_dir_chk()
{
    hp = h.homePath();
    hp.append("/.eepkeyer/");
    // Check to see if the ~/.eepkeyer directory exists
    if( !h.exists(hp)){
            h.mkdir(hp); //Creating missing .eepkeyer directory in users home dir
            msgBox.setText("New config file in ~/.eepkeyer");
            msgBox.exec();
        }
    // hp now contains a valid configuration file path ~/.eepkeyer
    return hp;
}

QString Dialog::cut_nums(QString q){
    // Check to see if any are enabled
    if(ui->zero_CB->isChecked()){
        q.replace(QRegExp("0"), ui->zero_LE->text());
    }
    if(ui->one_CB->isChecked()){
        q.replace(QRegExp("1"), ui->one_LE->text());
    }
    if(ui->two_CB->isChecked()){
        q.replace(QRegExp("2"), ui->two_LE->text());
    }
    if(ui->three_CB->isChecked()){
        q.replace(QRegExp("3"), ui->three_LE->text());
    }
    if(ui->four_CB->isChecked()){
        q.replace(QRegExp("4"), ui->four_LE->text());
    }
    if(ui->five_CB->isChecked()){
        q.replace(QRegExp("5"), ui->five_LE->text());
    }
    if(ui->six_CB->isChecked()){
        q.replace(QRegExp("6"), ui->six_LE->text());
    }
    if(ui->seven_CB->isChecked()){
        q.replace(QRegExp("7"), ui->seven_LE->text());
    }
    if(ui->eight_CB->isChecked()){
        q.replace(QRegExp("8"), ui->eight_LE->text());
    }
    if(ui->nine_CB->isChecked()){
        q.replace(QRegExp("9"), ui->nine_LE->text());
    }
return q;
}
